// this is a simple Arduino project used only for the purpose
// of renaming Bluetooth HM-10 module to a unique name to be 
// able to then pair to it;

#include <SoftwareSerial.h>

#define DEBUG true

// initialize communication port to BT module;
SoftwareSerial hm10_sw_serial(11, 10); // RX | TX

///////////////////////////////////////////////////////////////////////////////
//
// setup and loop functions;
//
///////////////////////////////////////////////////////////////////////////////
 
void setup()  
{
  // (1) default HW serial used usually for debug messages sent back to PC;
  Serial.begin(9600);
  Serial.println("\nWelcome to this sketch to rename your HM10 BT module!");
  
  // (2) initialize SW serial link to HM-10; baudrate for communicating 
  // with the HM-10 is 9600;
  hm10_sw_serial.begin(9600); 
  
  // (3) send AT commands to the HM-10 module; 
  // the format of these AT commands is described on the manufacturer's website:
  // https://www.deshide.com/product-details.html?pid=344835&_t=1665209850 (open 6.HM-10 datasheet.pdf supporting document)
  // NOTE: we do not need to append \r\n to each command like in the case of HC-05;
  send_command_to_hm10("AT",1000,DEBUG); 
  send_command_to_hm10("AT+VERSION",1000,DEBUG); 
  send_command_to_hm10("AT+NAMECris-HM10",1000,DEBUG); // <--- change Cris-HM10 to desired new name here!
  send_command_to_hm10("AT+NAME?",1000,DEBUG);
  
  Serial.println("\nDone!\n");
}

void loop()
{

}

///////////////////////////////////////////////////////////////////////////////
//
// functions related to handling the HM-10 bluetooth module;
//
///////////////////////////////////////////////////////////////////////////////

String send_command_to_hm10(String command, const int timeout, boolean debug)
{
  // Function used to send AT commands to HM-10.
  // command - the data/command to send; 
  // timeout - the time to wait for a response; 
  // debug - print to Serial window? (true = yes, false = no);
  // Return response from the HM-10 bluetooth module, if any; 
  
  String response = ""; 
  hm10_sw_serial.print(command); // send read character to the hc-05;
  
  long int time = millis();   
  while ( (time+timeout) > millis()) {
    while (hm10_sw_serial.available()) {      
      // HM-10 module has data; so display its output to the serial window; 
      char c = hm10_sw_serial.read(); // read the next character
      response+=c;
    }  
  }
  
  if (debug) {
    Serial.print(response);
    Serial.print("\n");
  }   
  return response;
}

String get_message_from_hm10()
{
  String msg = "";
  char a;
  while (hm10_sw_serial.available()) {
      a = hm10_sw_serial.read();
      msg += String(a);
  }
  return msg;
}