// this is a simple project which uses a computer 1 + Arduino board + BlueTooth HM-10 module
// to receive/send messages from/to a smartphone or another computer 2; it receives messages
// from smartphone and prints them via HW Serial to terminal on PC; 
// it takes from Serial what we type in terminal on PC and sends via HM-10 to
// the smartphone device; 

#include <SoftwareSerial.h>

#define DEBUG true

// initialize communication port to HM-10 BT module;
SoftwareSerial hm10_sw_serial(11, 10); // RX | TX

///////////////////////////////////////////////////////////////////////////////
//
// setup and loop functions;
//
///////////////////////////////////////////////////////////////////////////////
 
void setup()  
{
  // (1) default HW serial used usually for debug messages sent back to PC;
  Serial.begin(9600);
  Serial.println("\nWelcome to the ad-hoc BlueTooth Chat-Duplex System!");
  Serial.println("Please download a BlueTooth-to-Serial App for your smartphone.");
  Serial.println("Then, connect via BT to Arduino-hm10 and start chatting!");
  Serial.println("Enjoy!\n");
  
  // (2) initialize SW serial link to BT module; baudrate for communicating 
  // with the BT module is 9600;
  hm10_sw_serial.begin(9600); 
}

void loop()
{
  // (1) check if a message has been received via BT Module from smartphone;
  String msg = get_message_from_hm10();
  if (msg!="") {
    Serial.println(msg);
  }

  // (2) send message to the HW Serial Monitor to the BT Module
  if (Serial.available()) {
    hm10_sw_serial.write(Serial.read());
  }

  // (3) short break;
  delay(10);
}

///////////////////////////////////////////////////////////////////////////////
//
// functions related to handling the HM-10 bluetooth module;
//
///////////////////////////////////////////////////////////////////////////////

String send_command_to_hm10(String command, const int timeout, boolean debug)
{
  // Function used to send AT commands to HM-10.
  // command - the data/command to send; 
  // timeout - the time to wait for a response; 
  // debug - print to Serial window? (true = yes, false = no);
  // Return response from the HM-10 bluetooth module, if any; 
  
  String response = ""; 
  hm10_sw_serial.print(command); // send read character to the hm10;
  
  long int time = millis();   
  while ( (time+timeout) > millis()) {
    while (hm10_sw_serial.available()) {      
      // hm10 module has data; so display its output to the serial window; 
      char c = hm10_sw_serial.read(); // read the next character
      response+=c;
    }  
  }
  
  if (debug) {
    Serial.print(response);
    Serial.print("\n");
  }   
  return response;
}

String get_message_from_hm10()
{
  String msg = "";
  char a;
  while (hm10_sw_serial.available()) {
      a = hm10_sw_serial.read();
      msg += String(a);
  }
  return msg;
}